@@ -1,7 +1,7 @@
 #!/bin/sh
 
 #
-# Test mongocxx's Debian packaging scripts.
+# Test libmongocrypt's Debian packaging scripts.
 #
 # Supported/used environment variables:
 #   IS_PATCH    If "true", this is an Evergreen patch build.
@@ -45,10 +45,11 @@ sudo -E ./debootstrap.git/debootstrap unstable ./unstable-chroot/ http://cdn-aws
 cp -a libmongocrypt ./unstable-chroot/tmp/
 sudo chroot ./unstable-chroot /bin/bash -c "(set -o xtrace && \
   apt-get install -y build-essential git-buildpackage fakeroot debhelper cmake curl ca-certificates libssl-dev pkg-config libbson-dev && \
+  chown -R root:root /tmp/libmongocrypt && \
   cd /tmp/libmongocrypt && \
   git clean -fdx && \
   git reset --hard HEAD && \
-  cmake -P ./cmake/GetVersion.cmake > VERSION_CURRENT 2>&1 && \
+  cmake -DRELEASE_BRANCH_REF=origin/r1.5 -P ./cmake/GetVersion.cmake > VERSION_CURRENT 2>&1 && \
   git add --force VERSION_CURRENT && \
   git commit VERSION_CURRENT -m 'Set current version' && \
   LANG=C /bin/bash ./debian/build_snapshot.sh && \
