@@ -5,6 +5,9 @@ set -o errexit  # Exit the script with error if any of the commands fail
 echo "Setting up environment"
 . ./.evergreen/setup_environment.sh
 
+# Handle the circular dependency when testing with a real client.
+export MONGODB_CLIENT_ENCRYPTION_OVERRIDE="$(pwd)"
+
 # install node dependencies
 echo "Installing package dependencies (includes a static build)"
 . ./etc/build-static.sh
