@@ -93,14 +93,6 @@ module.exports = function (modules) {
       this._client = client;
       this._bson = options.bson || client.topology.bson;
       this._bypassEncryption = options.bypassAutoEncryption === true;
-      if (!this._bypassAutoEncryption) {
-        this._mongocryptdManager = new MongocryptdManager(options.extraOptions);
-        this._mongocryptdClient = new MongoClient(this._mongocryptdManager.uri, {
-          useNewUrlParser: true,
-          useUnifiedTopology: true,
-          serverSelectionTimeoutMS: 10000
-        });
-      }
 
       this._keyVaultNamespace = options.keyVaultNamespace || 'admin.datakeys';
       this._keyVaultClient = options.keyVaultClient || client;
@@ -116,6 +108,12 @@ module.exports = function (modules) {
           : this._bson.serialize(options.schemaMap);
       }
 
+      if (options.encryptedFieldsMap) {
+        mongoCryptOptions.encryptedFieldsMap = Buffer.isBuffer(options.encryptedFieldsMap)
+          ? options.encryptedFieldsMap
+          : this._bson.serialize(options.encryptedFieldsMap);
+      }
+
       if (options.kmsProviders) {
         mongoCryptOptions.kmsProviders = !Buffer.isBuffer(options.kmsProviders)
           ? this._bson.serialize(options.kmsProviders)
@@ -128,9 +126,46 @@ module.exports = function (modules) {
         mongoCryptOptions.logger = options.logger;
       }
 
+      if (options.extraOptions && options.extraOptions.cryptSharedLibPath) {
+        mongoCryptOptions.cryptSharedLibPath = options.extraOptions.cryptSharedLibPath;
+      }
+
+      if (options.bypassQueryAnalysis) {
+        mongoCryptOptions.bypassQueryAnalysis = options.bypassQueryAnalysis;
+      }
+
+      this._bypassMongocryptdAndCryptShared = this._bypassEncryption || options.bypassQueryAnalysis;
+
+      if (options.extraOptions && options.extraOptions.cryptSharedLibSearchPaths) {
+        // Only for driver testing
+        mongoCryptOptions.cryptSharedLibSearchPaths =
+          options.extraOptions.cryptSharedLibSearchPaths;
+      } else if (!this._bypassMongocryptdAndCryptShared) {
+        mongoCryptOptions.cryptSharedLibSearchPaths = ['$SYSTEM'];
+      }
+
       Object.assign(mongoCryptOptions, { cryptoCallbacks });
       this._mongocrypt = new mc.MongoCrypt(mongoCryptOptions);
       this._contextCounter = 0;
+
+      if (
+        options.extraOptions &&
+        options.extraOptions.cryptSharedLibRequired &&
+        !this.cryptSharedLibVersionInfo
+      ) {
+        throw new MongoError('`cryptSharedLibRequired` set but no crypt_shared library loaded');
+      }
+
+      // Only instantiate mongocryptd manager/client once we know for sure
+      // that we are not using the CSFLE shared library.
+      if (!this._bypassMongocryptdAndCryptShared && !this.cryptSharedLibVersionInfo) {
+        this._mongocryptdManager = new MongocryptdManager(options.extraOptions);
+        this._mongocryptdClient = new MongoClient(this._mongocryptdManager.uri, {
+          useNewUrlParser: true,
+          useUnifiedTopology: true,
+          serverSelectionTimeoutMS: 10000
+        });
+      }
     }
 
     /**
@@ -138,7 +173,7 @@ module.exports = function (modules) {
      * @param {Function} callback Invoked when the mongocryptd client either successfully connects or errors
      */
     init(callback) {
-      if (this._bypassEncryption) {
+      if (this._bypassMongocryptdAndCryptShared || this.cryptSharedLibVersionInfo) {
         return callback();
       }
       const _callback = (err, res) => {
@@ -170,7 +205,11 @@ module.exports = function (modules) {
      * @param {Function} callback Invoked when the mongocryptd client either successfully disconnects or errors
      */
     teardown(force, callback) {
-      this._mongocryptdClient.close(force, callback);
+      if (this._mongocryptdClient) {
+        this._mongocryptdClient.close(force, callback);
+      } else {
+        callback();
+      }
     }
 
     /**
@@ -220,6 +259,8 @@ module.exports = function (modules) {
       const stateMachine = new StateMachine({
         bson,
         ...options,
+        promoteValues: false,
+        promoteLongs: false,
         proxyOptions: this._proxyOptions,
         tlsOptions: this._tlsOptions
       });
@@ -281,6 +322,15 @@ module.exports = function (modules) {
     async askForKMSCredentials() {
       return this._onKmsProviderRefresh ? this._onKmsProviderRefresh() : {};
     }
+
+    /**
+     * Return the current libmongocrypt's CSFLE shared library version
+     * as `{ version: bigint, versionStr: string }`, or `null` if no CSFLE
+     * shared library was loaded.
+     */
+    get cryptSharedLibVersionInfo() {
+      return this._mongocrypt.cryptSharedLibVersionInfo;
+    }
   }
 
   return { AutoEncrypter };
@@ -320,6 +370,9 @@ function decorateDecryptionResult(decrypted, original, bson, isTopLevelDecorateC
         });
       }
       decrypted[decryptedKeys].push(k);
+      // Do not recurse into this decrypted value. It could be a subdocument/array,
+      // in which case there is no original value associated with its subfields.
+      continue;
     }
 
     decorateDecryptionResult(decrypted[k], originalValue, bson, false);
