@@ -170,6 +170,10 @@ module.exports = function (modules) {
      *   keyAltNames: [ 'mySpecialKey' ]
      * });
      */
+    createKey(provider, options, callback) {
+      return this.createDataKey(provider, options, callback);
+    }
+
     createDataKey(provider, options, callback) {
       if (typeof options === 'function') {
         callback = options;
@@ -202,8 +206,16 @@ module.exports = function (modules) {
         });
       }
 
+      let keyMaterial = undefined;
+      if (options.keyMaterial) {
+        keyMaterial = bson.serialize({ keyMaterial: options.keyMaterial });
+      }
+
       const dataKeyBson = bson.serialize(dataKey);
-      const context = this._mongoCrypt.makeDataKeyContext(dataKeyBson, { keyAltNames });
+      const context = this._mongoCrypt.makeDataKeyContext(dataKeyBson, {
+        keyAltNames,
+        keyMaterial
+      });
       const stateMachine = new StateMachine({
         bson,
         proxyOptions: this._proxyOptions,
@@ -235,6 +247,74 @@ module.exports = function (modules) {
       });
     }
 
+    rewrapManyDataKey(filter, options, callback) {
+      if (typeof options === 'function') {
+        callback = options;
+        options = {};
+      }
+
+      const bson = this._bson;
+
+      let keyEncryptionKeyBson = undefined;
+      if (options) {
+        const keyEncryptionKey = Object.assign({ provider: options.provider }, options.masterKey);
+        keyEncryptionKeyBson = bson.serialize(keyEncryptionKey);
+      } else {
+        // Always make sure `options` is an object below.
+        options = {};
+      }
+      const filterBson = bson.serialize(filter);
+      const context = this._mongoCrypt.makeRewrapManyDataKeyContext(
+        filterBson,
+        keyEncryptionKeyBson
+      );
+      const stateMachine = new StateMachine({
+        bson,
+        proxyOptions: this._proxyOptions,
+        tlsOptions: this._tlsOptions,
+        session: options.session
+      });
+
+      return promiseOrCallback(callback, cb => {
+        stateMachine.execute(this, context, (err, dataKey) => {
+          if (err) {
+            cb(err, null);
+            return;
+          }
+
+          const dbName = databaseNamespace(this._keyVaultNamespace);
+          const collectionName = collectionNamespace(this._keyVaultNamespace);
+          const replacements = dataKey.v.map(key => ({
+            replaceOne: {
+              filter: { _id: key._id },
+              replacement: key
+            }
+          }));
+
+          this._keyVaultClient
+            .db(dbName)
+            .collection(collectionName)
+            .bulkWrite(
+              replacements,
+              {
+                writeConcern: { w: 'majority' },
+                session: options.session
+              },
+              (err, result) => {
+                if (err) {
+                  cb(err, null);
+                  return;
+                }
+
+                cb(null, {
+                  bulkWriteResult: result
+                });
+              }
+            );
+        });
+      });
+    }
+
     /**
      * @callback ClientEncryption~encryptCallback
      * @param {Error} [err] If present, indicates an error that occurred in the process of encryption
@@ -249,7 +329,7 @@ module.exports = function (modules) {
      * @param {object} options
      * @param {ClientEncryption~dataKeyId} [options.keyId] The id of the Binary dataKey to use for encryption
      * @param {string} [options.keyAltName] A unique string name corresponding to an already existing dataKey.
-     * @param {} options.algorithm The algorithm to use for encryption. Must be either `'AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic'` or `AEAD_AES_256_CBC_HMAC_SHA_512-Random'`
+     * @param {} [options.algorithm] The algorithm to use for encryption. Must be either `'AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic'`, `'AEAD_AES_256_CBC_HMAC_SHA_512-Random'`, `'Indexed'` or `'Unindexed'`
      * @param {ClientEncryption~encryptCallback} [callback] Optional callback to invoke when value is encrypted
      * @returns {Promise|void} If no callback is provided, returns a Promise that either resolves with the encrypted value, or rejects with an error. If a callback is provided, returns nothing.
      *
@@ -285,6 +365,9 @@ module.exports = function (modules) {
       if (options.keyId) {
         contextOptions.keyId = options.keyId.buffer;
       }
+      if (options.indexKeyId) {
+        contextOptions.indexKeyId = options.indexKeyId.buffer;
+      }
       if (options.keyAltName) {
         const keyAltName = options.keyAltName;
         if (options.keyId) {
