@@ -404,7 +404,11 @@ module.exports = function (modules) {
 
       client
         .db(dbName)
-        .listCollections(filter)
+        .listCollections(filter, {
+          promoteLongs: false,
+          promoteValues: false,
+          session: this.options.session
+        })
         .toArray((err, collections) => {
           if (err) {
             callback(err, null);
@@ -428,15 +432,15 @@ module.exports = function (modules) {
      */
     markCommand(client, ns, command, callback) {
       const bson = this.bson;
+      const options = { promoteLongs: false, promoteValues: false };
       const dbName = databaseNamespace(ns);
-      const rawCommand = bson.deserialize(command, { promoteLongs: false, promoteValues: false });
+      const rawCommand = bson.deserialize(command, options);
 
-      client.db(dbName).command(rawCommand, (err, response) => {
+      client.db(dbName).command(rawCommand, options, (err, response) => {
         if (err) {
           callback(err, null);
           return;
         }
-
         callback(err, bson.serialize(response, this.options));
       });
     }
@@ -460,7 +464,7 @@ module.exports = function (modules) {
       client
         .db(dbName)
         .collection(collectionName, { readConcern: { level: 'majority' } })
-        .find(filter)
+        .find(filter, { session: this.options.session })
         .toArray((err, keys) => {
           if (err) {
             callback(err, null);
