@@ -81,6 +81,21 @@ std::string errorStringFromStatus(mongocrypt_ctx_t* context) {
     return errorMessage;
 }
 
+template<typename E>
+E strToEnumValue(
+    Env env,
+    const std::string& str,
+    const char* option_name,
+    const std::initializer_list<std::pair<const char*, E>>& values) {
+    for (const auto& candidate : values) {
+        if (candidate.first == str) {
+            return candidate.second;
+        }
+    }
+    throw Error::New(env,
+        std::string("invalid enum value: '") + str + "' for " + option_name);
+}
+
 }  // anonymous namespace
 
 Function MongoCrypt::Init(Napi::Env env) {
@@ -93,7 +108,9 @@ Function MongoCrypt::Init(Napi::Env env) {
                     InstanceMethod("makeDecryptionContext", &MongoCrypt::MakeDecryptionContext),
                     InstanceMethod("makeExplicitDecryptionContext", &MongoCrypt::MakeExplicitDecryptionContext),
                     InstanceMethod("makeDataKeyContext", &MongoCrypt::MakeDataKeyContext),
-                    InstanceAccessor("status", &MongoCrypt::Status, nullptr)
+                    InstanceMethod("makeRewrapManyDataKeyContext", &MongoCrypt::MakeRewrapManyDataKeyContext),
+                    InstanceAccessor("status", &MongoCrypt::Status, nullptr),
+                    InstanceAccessor("cryptSharedLibVersionInfo", &MongoCrypt::CryptSharedLibVersionInfo, nullptr)
                   });
 }
 
@@ -145,66 +162,57 @@ static void MaybeSetCryptoHookErrorStatus(Value result, mongocrypt_status_t *sta
     );
 }
 
+static bool aes_256_generic_hook (MongoCrypt* mongoCrypt, mongocrypt_binary_t *key, mongocrypt_binary_t *iv, mongocrypt_binary_t *in, mongocrypt_binary_t *out, uint32_t *bytes_written, mongocrypt_status_t *status, Function hook) {
+    Env env = mongoCrypt->Env();
+    HandleScope scope(env);
+
+    Uint8Array keyBuffer = BufferFromBinary(env, key);
+    Uint8Array ivBuffer = BufferFromBinary(env, iv);
+    Uint8Array inBuffer = BufferFromBinary(env, in);
+    Uint8Array outBuffer = BufferWithLengthOf(env, out);
+
+    Value result;
+    try {
+        result = hook.Call(std::initializer_list<napi_value>
+            { keyBuffer, ivBuffer, inBuffer, outBuffer });
+    } catch (...) {
+        return false;
+    }
+
+    if (!result.IsNumber()) {
+        MaybeSetCryptoHookErrorStatus(result, status);
+        return false;
+    }
+
+    *bytes_written = result.ToNumber().Uint32Value();
+    CopyBufferData(out, outBuffer, *bytes_written);
+    return true;
+}
+
 bool MongoCrypt::setupCryptoHooks() {
     auto aes_256_cbc_encrypt =
         [](void *ctx, mongocrypt_binary_t *key, mongocrypt_binary_t *iv, mongocrypt_binary_t *in, mongocrypt_binary_t *out, uint32_t *bytes_written, mongocrypt_status_t *status) -> bool {
-            MongoCrypt* mongoCrypt = static_cast<MongoCrypt*>(ctx);
-            Napi::Env env = mongoCrypt->Env();
-            HandleScope scope(env);
-            Function hook = mongoCrypt->GetCallback("aes256CbcEncryptHook");
-
-            Uint8Array keyBuffer = BufferFromBinary(env, key);
-            Uint8Array ivBuffer = BufferFromBinary(env, iv);
-            Uint8Array inBuffer = BufferFromBinary(env, in);
-            Uint8Array outBuffer = BufferWithLengthOf(env, out);
-
-            Napi::Value result;
-            try {
-                result = hook.Call(std::initializer_list<napi_value>
-                    { keyBuffer, ivBuffer, inBuffer, outBuffer });
-            } catch (...) {
-                return false;
-            }
-
-            if (!result.IsNumber()) {
-                MaybeSetCryptoHookErrorStatus(result, status);
-                return false;
-            }
-
-            *bytes_written = result.ToNumber().Uint32Value();
-            CopyBufferData(out, outBuffer, *bytes_written);
-            return true;
-        };
+        MongoCrypt* mc = static_cast<MongoCrypt*>(ctx);
+        return aes_256_generic_hook(mc, key, iv, in, out, bytes_written, status, mc->GetCallback("aes256CbcEncryptHook"));
+    };
 
     auto aes_256_cbc_decrypt =
         [](void *ctx, mongocrypt_binary_t *key, mongocrypt_binary_t *iv, mongocrypt_binary_t *in, mongocrypt_binary_t *out, uint32_t *bytes_written, mongocrypt_status_t *status) -> bool {
-            MongoCrypt* mongoCrypt = static_cast<MongoCrypt*>(ctx);
-            Napi::Env env = mongoCrypt->Env();
-            HandleScope scope(env);
-            Function hook = mongoCrypt->GetCallback("aes256CbcDecryptHook");
+        MongoCrypt* mc = static_cast<MongoCrypt*>(ctx);
+        return aes_256_generic_hook(mc, key, iv, in, out, bytes_written, status, mc->GetCallback("aes256CbcDecryptHook"));
+    };
 
-            Uint8Array keyBuffer = BufferFromBinary(env, key);
-            Uint8Array ivBuffer = BufferFromBinary(env, iv);
-            Uint8Array inBuffer = BufferFromBinary(env, in);
-            Uint8Array outBuffer = BufferWithLengthOf(env, out);
+    auto aes_256_ctr_encrypt =
+        [](void *ctx, mongocrypt_binary_t *key, mongocrypt_binary_t *iv, mongocrypt_binary_t *in, mongocrypt_binary_t *out, uint32_t *bytes_written, mongocrypt_status_t *status) -> bool {
+        MongoCrypt* mc = static_cast<MongoCrypt*>(ctx);
+        return aes_256_generic_hook(mc, key, iv, in, out, bytes_written, status, mc->GetCallback("aes256CtrEncryptHook"));
+    };
 
-            Napi::Value result;
-            try {
-                result = hook.Call(std::initializer_list<napi_value>
-                    { keyBuffer, ivBuffer, inBuffer, outBuffer });
-            } catch (...) {
-                return false;
-            }
-
-            if (!result.IsNumber()) {
-                MaybeSetCryptoHookErrorStatus(result, status);
-                return false;
-            }
-
-            *bytes_written = result.ToNumber().Uint32Value();
-            CopyBufferData(out, outBuffer, *bytes_written);
-            return true;
-        };
+    auto aes_256_ctr_decrypt =
+        [](void *ctx, mongocrypt_binary_t *key, mongocrypt_binary_t *iv, mongocrypt_binary_t *in, mongocrypt_binary_t *out, uint32_t *bytes_written, mongocrypt_status_t *status) -> bool {
+        MongoCrypt* mc = static_cast<MongoCrypt*>(ctx);
+        return aes_256_generic_hook(mc, key, iv, in, out, bytes_written, status, mc->GetCallback("aes256CtrDecryptHook"));
+    };
 
     auto random =
         [](void *ctx, mongocrypt_binary_t *out, uint32_t count, mongocrypt_status_t *status) -> bool {
@@ -342,20 +350,27 @@ bool MongoCrypt::setupCryptoHooks() {
             return true;
         };
 
-    // Added after `mongocrypt_setopt_crypto_hooks`, they should be treated as the same during configuration
-    if (!mongocrypt_setopt_crypto_hook_sign_rsaes_pkcs1_v1_5(_mongo_crypt.get(), sign_rsa_sha256, this)) {
-        return false;
-    }
-
-    return mongocrypt_setopt_crypto_hooks(_mongo_crypt.get(),
+    if (!mongocrypt_setopt_crypto_hooks(_mongo_crypt.get(),
         aes_256_cbc_encrypt,
         aes_256_cbc_decrypt,
         random,
         hmac_sha_512,
         hmac_sha_256,
         sha_256,
-        this
-    );
+        this)) {
+        return false;
+    }
+
+    // Added after `mongocrypt_setopt_crypto_hooks`, they should be treated as the same during configuration
+    if (!mongocrypt_setopt_crypto_hook_sign_rsaes_pkcs1_v1_5(_mongo_crypt.get(), sign_rsa_sha256, this)) {
+        return false;
+    }
+
+    if (!mongocrypt_setopt_aes_256_ctr(_mongo_crypt.get(), aes_256_ctr_encrypt, aes_256_ctr_decrypt, this)) {
+        return false;
+    }
+
+    return true;
 }
 
 MongoCrypt::MongoCrypt(const CallbackInfo& info)
@@ -394,6 +409,20 @@ MongoCrypt::MongoCrypt(const CallbackInfo& info)
         }
     }
 
+    if (options.Has("encryptedFieldsMap")) {
+        Napi::Value encryptedFieldsMapBuffer = options["encryptedFieldsMap"];
+
+        if (!encryptedFieldsMapBuffer.IsBuffer()) {
+            throw TypeError::New(Env(), "Option `encryptedFieldsMap` must be a Buffer");
+        }
+
+        std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> encryptedFieldsMapBinary(
+            BufferToBinary(encryptedFieldsMapBuffer.As<Uint8Array>()));
+        if (!mongocrypt_setopt_encrypted_field_config_map(_mongo_crypt.get(), encryptedFieldsMapBinary.get())) {
+            throw TypeError::New(Env(), errorStringFromStatus(_mongo_crypt.get()));
+        }
+    }
+
     if (options.Has("logger")) {
         SetCallback("logger", options["logger"]);
         if (!mongocrypt_setopt_log_handler(
@@ -407,6 +436,8 @@ MongoCrypt::MongoCrypt(const CallbackInfo& info)
 
         SetCallback("aes256CbcEncryptHook", cryptoCallbacks["aes256CbcEncryptHook"]);
         SetCallback("aes256CbcDecryptHook", cryptoCallbacks["aes256CbcDecryptHook"]);
+        SetCallback("aes256CtrEncryptHook", cryptoCallbacks["aes256CtrEncryptHook"]);
+        SetCallback("aes256CtrDecryptHook", cryptoCallbacks["aes256CtrDecryptHook"]);
         SetCallback("randomHook", cryptoCallbacks["randomHook"]);
         SetCallback("hmacSha512Hook", cryptoCallbacks["hmacSha512Hook"]);
         SetCallback("hmacSha256Hook", cryptoCallbacks["hmacSha256Hook"]);
@@ -418,15 +449,50 @@ MongoCrypt::MongoCrypt(const CallbackInfo& info)
         }
     }
 
+    if (options.Has("cryptSharedLibSearchPaths")) {
+        Napi::Value search_paths_v = options["cryptSharedLibSearchPaths"];
+        if (!search_paths_v.IsArray()) {
+            throw TypeError::New(Env(), "Option `cryptSharedLibSearchPaths` must be an array");
+        }
+        Array search_paths = search_paths_v.As<Array>();
+        for (uint32_t i = 0; i < search_paths.Length(); i++) {
+            mongocrypt_setopt_append_crypt_shared_lib_search_path(
+                _mongo_crypt.get(),
+                search_paths.Get(i).ToString().Utf8Value().c_str());
+        }
+    }
+
+    if (options.Has("cryptSharedLibPath")) {
+        mongocrypt_setopt_set_crypt_shared_lib_path_override(
+            _mongo_crypt.get(),
+            options.Get("cryptSharedLibPath").ToString().Utf8Value().c_str());
+    }
+
+    if (options.Get("bypassQueryAnalysis").ToBoolean()) {
+        mongocrypt_setopt_bypass_query_analysis(_mongo_crypt.get());
+    }
+
     mongocrypt_setopt_use_need_kms_credentials_state(_mongo_crypt.get());
 
-    // initialize afer all options are set, but after `MongoCrypt` instance is created so we can
-    // optionally pass the instance to the logging function.
+    // Initialize after all options are set.
     if (!mongocrypt_init(_mongo_crypt.get())) {
         throw TypeError::New(Env(), errorStringFromStatus(_mongo_crypt.get()));
     }
 }
 
+Value MongoCrypt::CryptSharedLibVersionInfo(const CallbackInfo& info) {
+    uint64_t version_numeric = mongocrypt_crypt_shared_lib_version(_mongo_crypt.get());
+    const char* version_string = mongocrypt_crypt_shared_lib_version_string(_mongo_crypt.get(), nullptr);
+    if (version_string == nullptr) {
+        return Env().Null();
+    }
+
+    Object ret = Object::New(Env());
+    ret["version"] = BigInt::New(Env(), version_numeric);
+    ret["versionStr"] = String::New(Env(), version_string);
+    return ret;
+}
+
 Value MongoCrypt::Status(const CallbackInfo& info) {
     std::unique_ptr<mongocrypt_status_t, MongoCryptStatusDeleter> status(mongocrypt_status_new());
     mongocrypt_status(_mongo_crypt.get(), status.get());
@@ -494,8 +560,37 @@ Value MongoCrypt::MakeExplicitEncryptionContext(const CallbackInfo& info) {
         if (options.Has("algorithm")) {
             std::string algorithm = options.Get("algorithm").ToString();
             if (!mongocrypt_ctx_setopt_algorithm(
-                    context.get(), const_cast<char*>(algorithm.c_str()), algorithm.size())) {
+                    context.get(), algorithm.c_str(), algorithm.size())) {
+                throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+            }
+        }
 
+        if (options.Has("contentionFactor")) {
+            Napi::Value contention_factor_value = options["contentionFactor"];
+            int64_t contention_factor = contention_factor_value.IsBigInt() ?
+                contention_factor_value.As<BigInt>().Int64Value(nullptr) :
+                contention_factor_value.ToNumber().Int64Value();
+            if (!mongocrypt_ctx_setopt_contention_factor(context.get(), contention_factor)) {
+                throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+            }
+        }
+
+        if (options.Has("queryType")) {
+            std::string query_type_str = options.Get("queryType").ToString();
+            if (!mongocrypt_ctx_setopt_query_type(context.get(), query_type_str.data(), -1)) {
+                throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+            }
+        }
+
+        if (options.Has("indexKeyId")) {
+            Napi::Value indexKeyId = options["indexKeyId"];
+
+            if (!indexKeyId.IsBuffer()) {
+                throw TypeError::New(Env(), "`indexKeyId` must be a Buffer");
+            }
+
+            std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> binary(BufferToBinary(indexKeyId.As<Uint8Array>()));
+            if (!mongocrypt_ctx_setopt_index_key_id(context.get(), binary.get())) {
                 throw TypeError::New(Env(), errorStringFromStatus(context.get()));
             }
         }
@@ -583,6 +678,23 @@ Value MongoCrypt::MakeDataKeyContext(const CallbackInfo& info) {
         }
     }
 
+    if (options.Has("keyMaterial")) {
+        Napi::Value keyMaterial = options["keyMaterial"];
+
+        if (!keyMaterial.IsUndefined()) {
+            if (!keyMaterial.IsBuffer()) {
+                // We should never get here
+                throw TypeError::New(Env(), "Serialized keyMaterial must be a Buffer");
+            }
+
+            std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> binary(
+                BufferToBinary(keyMaterial.As<Uint8Array>()));
+            if (!mongocrypt_ctx_setopt_key_material(context.get(), binary.get())) {
+                throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+            }
+        }
+    }
+
     if (!mongocrypt_ctx_datakey_init(context.get())) {
         throw TypeError::New(Env(), errorStringFromStatus(context.get()));
     }
@@ -590,6 +702,33 @@ Value MongoCrypt::MakeDataKeyContext(const CallbackInfo& info) {
     return MongoCryptContext::NewInstance(Env(), std::move(context));
 }
 
+Value MongoCrypt::MakeRewrapManyDataKeyContext(const CallbackInfo& info) {
+    Napi::Value filter_buffer = info[0];
+    if (!filter_buffer.IsBuffer()) {
+        throw TypeError::New(Env(), "Parameter `options` must be a Buffer");
+    }
+
+    std::unique_ptr<mongocrypt_ctx_t, MongoCryptContextDeleter> context(
+        mongocrypt_ctx_new(_mongo_crypt.get()));
+
+    Napi::Value key_encryption_key = info[1];
+    if (key_encryption_key.IsBuffer()) {
+        std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> key_binary(
+            BufferToBinary(key_encryption_key.As<Uint8Array>()));
+        if (!mongocrypt_ctx_setopt_key_encryption_key(context.get(), key_binary.get())) {
+            throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+        }
+    }
+
+    std::unique_ptr<mongocrypt_binary_t, MongoCryptBinaryDeleter> filter_binary(
+        BufferToBinary(filter_buffer.As<Uint8Array>()));
+    if (!mongocrypt_ctx_rewrap_many_datakey_init(context.get(), filter_binary.get())) {
+        throw TypeError::New(Env(), errorStringFromStatus(context.get()));
+    }
+
+    return MongoCryptContext::NewInstance(Env(), std::move(context));
+}
+
 // Store callbacks as nested properties on the MongoCrypt binding object
 // itself, and use these helpers to do so. Storing them as JS engine
 // References is a big memory leak footgun.
