@@ -1,12 +1,14 @@
 'use strict';
 
 const fs = require('fs');
+const path = require('path');
 const BSON = require('bson');
 const EJSON = require('bson').EJSON;
 const sinon = require('sinon');
 const mongodb = require('mongodb');
 const requirements = require('./requirements.helper');
 const MongoNetworkTimeoutError = mongodb.MongoNetworkTimeoutError || mongodb.MongoTimeoutError;
+const MongoError = mongodb.MongoError;
 const stateMachine = require('../lib/stateMachine')({ mongodb });
 const StateMachine = stateMachine.StateMachine;
 const MongocryptdManager = require('../lib/mongocryptdManager').MongocryptdManager;
@@ -16,13 +18,34 @@ const expect = chai.expect;
 chai.use(require('chai-subset'));
 chai.use(require('sinon-chai'));
 
+const sharedLibrarySuffix =
+  process.platform === 'win32' ? 'dll' : process.platform === 'darwin' ? 'dylib' : 'so';
+let sharedLibraryStub = path.resolve(
+  __dirname,
+  '..',
+  '..',
+  '..',
+  `mongo_crypt_v1.${sharedLibrarySuffix}`
+);
+if (!fs.existsSync(sharedLibraryStub)) {
+  sharedLibraryStub = path.resolve(
+    __dirname,
+    '..',
+    'deps',
+    'tmp',
+    'libmongocrypt-build',
+    ...(process.platform === 'win32' ? ['RelWithDebInfo'] : []),
+    `mongo_crypt_v1.${sharedLibrarySuffix}`
+  );
+}
+
 function readExtendedJsonToBuffer(path) {
   const ejson = EJSON.parse(fs.readFileSync(path, 'utf8'));
   return BSON.serialize(ejson);
 }
 
 function readHttpResponse(path) {
-  let data = fs.readFileSync(path, 'utf8').toString();
+  let data = fs.readFileSync(path, 'utf8');
   data = data.split('\n').join('\r\n');
   return Buffer.from(data, 'utf8');
 }
@@ -147,6 +170,9 @@ describe('AutoEncrypter', function () {
 
     it('should decrypt mock data and mark decrypted items if enabled for testing', function (done) {
       const input = readExtendedJsonToBuffer(`${__dirname}/data/encrypted-document.json`);
+      const nestedInput = readExtendedJsonToBuffer(
+        `${__dirname}/data/encrypted-document-nested.json`
+      );
       const client = new MockClient();
       const mc = new AutoEncrypter(client, {
         keyVaultNamespace: 'admin.datakeys',
@@ -168,7 +194,17 @@ describe('AutoEncrypter', function () {
           if (err) return done(err);
           expect(decrypted).to.eql({ a: [null, 1, { c: new BSON.Binary('foo', 1) }] });
           expect(decrypted).to.not.have.property(Symbol.for('@@mdb.decryptedKeys'));
-          done();
+
+          // The same, but with nested data inside the decrypted input
+          mc.decrypt(nestedInput, (err, decrypted) => {
+            if (err) return done(err);
+            expect(decrypted).to.eql({ nested: { x: { y: 1234 } } });
+            expect(decrypted[Symbol.for('@@mdb.decryptedKeys')]).to.eql(['nested']);
+            expect(decrypted.nested).to.not.have.property(Symbol.for('@@mdb.decryptedKeys'));
+            expect(decrypted.nested.x).to.not.have.property(Symbol.for('@@mdb.decryptedKeys'));
+            expect(decrypted.nested.x.y).to.not.have.property(Symbol.for('@@mdb.decryptedKeys'));
+            done();
+          });
         });
       });
     });
@@ -260,6 +296,48 @@ describe('AutoEncrypter', function () {
         done();
       });
     });
+
+    // TODO(NODE-4089): Enable test once https://github.com/mongodb/libmongocrypt/pull/263 is done
+    it.skip('should encrypt mock data when using the crypt_shared library', function (done) {
+      const client = new MockClient();
+      const mc = new AutoEncrypter(client, {
+        keyVaultNamespace: 'admin.datakeys',
+        logger: () => {},
+        kmsProviders: {
+          aws: {}
+        },
+        async onKmsProviderRefresh() {
+          return { aws: { accessKeyId: 'example', secretAccessKey: 'example' } };
+        },
+        extraOptions: {
+          cryptSharedLibPath: sharedLibraryStub
+        }
+      });
+
+      expect(mc).to.not.have.property('_mongocryptdManager');
+      expect(mc).to.not.have.property('_mongocryptdClient');
+
+      mc.encrypt('test.test', TEST_COMMAND, (err, encrypted) => {
+        if (err) return done(err);
+        const expected = EJSON.parse(
+          JSON.stringify({
+            find: 'test',
+            filter: {
+              ssn: {
+                $binary: {
+                  base64:
+                    'AWFhYWFhYWFhYWFhYWFhYWECRTOW9yZzNDn5dGwuqsrJQNLtgMEKaujhs9aRWRp+7Yo3JK8N8jC8P0Xjll6C1CwLsE/iP5wjOMhVv1KMMyOCSCrHorXRsb2IKPtzl2lKTqQ=',
+                  subType: '6'
+                }
+              }
+            }
+          })
+        );
+
+        expect(encrypted).to.containSubset(expected);
+        done();
+      });
+    });
   });
 
   describe('logging', function () {
@@ -331,6 +409,8 @@ describe('AutoEncrypter', function () {
         }
       });
 
+      expect(this.mc).to.have.property('cryptSharedLibVersionInfo', null);
+
       const localMcdm = this.mc._mongocryptdManager;
       sandbox.spy(localMcdm, 'spawn');
 
@@ -362,6 +442,7 @@ describe('AutoEncrypter', function () {
           local: { key: Buffer.alloc(96) }
         }
       });
+      expect(this.mc).to.have.property('cryptSharedLibVersionInfo', null);
 
       const localMcdm = this.mc._mongocryptdManager;
       this.mc.init(err => {
@@ -398,6 +479,7 @@ describe('AutoEncrypter', function () {
           local: { key: Buffer.alloc(96) }
         }
       });
+      expect(this.mc).to.have.property('cryptSharedLibVersionInfo', null);
 
       const localMcdm = this.mc._mongocryptdManager;
       this.mc.init(err => {
@@ -434,6 +516,7 @@ describe('AutoEncrypter', function () {
           local: { key: Buffer.alloc(96) }
         }
       });
+      expect(this.mc).to.have.property('cryptSharedLibVersionInfo', null);
 
       const localMcdm = this.mc._mongocryptdManager;
       this.mc.init(err => {
@@ -462,6 +545,7 @@ describe('AutoEncrypter', function () {
           mongocryptdURI: 'mongodb://something.invalid:27020/'
         }
       });
+      expect(this.mc).to.have.property('cryptSharedLibVersionInfo', null);
 
       sandbox.stub(MongocryptdManager.prototype, 'spawn').callsFake(callback => {
         callback();
@@ -469,7 +553,7 @@ describe('AutoEncrypter', function () {
 
       this.mc.init(err => {
         expect(err).to.exist;
-        expect(err).to.match(/Unable to connect to `mongocryptd`/);
+        expect(err).to.be.instanceOf(MongoError);
         done();
       });
     });
@@ -497,7 +581,7 @@ describe('AutoEncrypter', function () {
       }
     });
 
-    ['mongocryptdBypassSpawn', 'bypassAutoEncryption'].forEach(opt => {
+    ['mongocryptdBypassSpawn', 'bypassAutoEncryption', 'bypassQueryAnalysis'].forEach(opt => {
       const encryptionOptions = {
         keyVaultNamespace: 'admin.datakeys',
         logger: () => {},
@@ -508,14 +592,15 @@ describe('AutoEncrypter', function () {
         extraOptions: {
           mongocryptdBypassSpawn: opt === 'mongocryptdBypassSpawn'
         },
-        bypassAutoEncryption: opt === 'bypassAutoEncryption'
+        bypassAutoEncryption: opt === 'bypassAutoEncryption',
+        bypassQueryAnalysis: opt === 'bypassQueryAnalysis'
       };
 
       it(`should not spawn mongocryptd on startup if ${opt} is true`, function (done) {
         const client = new MockClient();
         this.mc = new AutoEncrypter(client, encryptionOptions);
 
-        const localMcdm = this.mc._mongocryptdManager;
+        const localMcdm = this.mc._mongocryptdManager || { spawn: () => {} };
         sandbox.spy(localMcdm, 'spawn');
 
         this.mc.init(err => {
@@ -524,22 +609,58 @@ describe('AutoEncrypter', function () {
           done();
         });
       });
+    });
 
-      it('should not spawn a mongocryptd or retry on a server selection error if mongocryptdBypassSpawn: true', function (done) {
-        let called = false;
-        const timeoutError = new MongoNetworkTimeoutError('msg');
-        StateMachine.prototype.markCommand.callsFake((client, ns, filter, callback) => {
-          if (!called) {
-            called = true;
-            callback(timeoutError);
-            return;
-          }
+    it('should not spawn a mongocryptd or retry on a server selection error if mongocryptdBypassSpawn: true', function (done) {
+      let called = false;
+      const timeoutError = new MongoNetworkTimeoutError('msg');
+      StateMachine.prototype.markCommand.callsFake((client, ns, filter, callback) => {
+        if (!called) {
+          called = true;
+          callback(timeoutError);
+          return;
+        }
 
-          callback(null, MOCK_MONGOCRYPTD_RESPONSE);
+        callback(null, MOCK_MONGOCRYPTD_RESPONSE);
+      });
+
+      const client = new MockClient();
+      this.mc = new AutoEncrypter(client, {
+        keyVaultNamespace: 'admin.datakeys',
+        logger: () => {},
+        kmsProviders: {
+          aws: { accessKeyId: 'example', secretAccessKey: 'example' },
+          local: { key: Buffer.alloc(96) }
+        },
+        extraOptions: {
+          mongocryptdBypassSpawn: true
+        }
+      });
+
+      const localMcdm = this.mc._mongocryptdManager;
+      sandbox.spy(localMcdm, 'spawn');
+
+      this.mc.init(err => {
+        expect(err).to.not.exist;
+        expect(localMcdm.spawn).to.not.have.been.called;
+
+        this.mc.encrypt('test.test', TEST_COMMAND, (err, response) => {
+          expect(localMcdm.spawn).to.not.have.been.called;
+          expect(response).to.not.exist;
+          expect(err).to.equal(timeoutError);
+          done();
         });
+      });
+    });
+  });
 
-        const client = new MockClient();
-        this.mc = new AutoEncrypter(client, {
+  describe('crypt_shared library', function () {
+    it('should fail if no library can be found in the search path and cryptSharedLibRequired is set', function () {
+      // NB: This test has to be run before the tests/without having previously
+      // loaded a CSFLE shared library below to get the right error path.
+      const client = new MockClient();
+      try {
+        new AutoEncrypter(client, {
           keyVaultNamespace: 'admin.datakeys',
           logger: () => {},
           kmsProviders: {
@@ -547,25 +668,66 @@ describe('AutoEncrypter', function () {
             local: { key: Buffer.alloc(96) }
           },
           extraOptions: {
-            mongocryptdBypassSpawn: true
+            cryptSharedLibSearchPaths: ['/nonexistent'],
+            cryptSharedLibRequired: true
           }
         });
+        expect.fail('missed exception');
+      } catch (err) {
+        expect(err.message).to.include(
+          '`cryptSharedLibRequired` set but no crypt_shared library loaded'
+        );
+      }
+    });
 
-        const localMcdm = this.mc._mongocryptdManager;
-        sandbox.spy(localMcdm, 'spawn');
-
-        this.mc.init(err => {
-          expect(err).to.not.exist;
-          expect(localMcdm.spawn).to.not.have.been.called;
-
-          this.mc.encrypt('test.test', TEST_COMMAND, (err, response) => {
-            expect(localMcdm.spawn).to.not.have.been.called;
-            expect(response).to.not.exist;
-            expect(err).to.equal(timeoutError);
-            done();
-          });
-        });
+    it('should load a shared library by specifying its path', function (done) {
+      const client = new MockClient();
+      this.mc = new AutoEncrypter(client, {
+        keyVaultNamespace: 'admin.datakeys',
+        logger: () => {},
+        kmsProviders: {
+          aws: { accessKeyId: 'example', secretAccessKey: 'example' },
+          local: { key: Buffer.alloc(96) }
+        },
+        extraOptions: {
+          cryptSharedLibPath: sharedLibraryStub
+        }
       });
+
+      expect(this.mc).to.not.have.property('_mongocryptdManager');
+      expect(this.mc).to.not.have.property('_mongocryptdClient');
+      expect(this.mc).to.have.deep.property('cryptSharedLibVersionInfo', {
+        // eslint-disable-next-line no-undef
+        version: BigInt(0x000600020001000),
+        versionStr: 'stubbed-crypt_shared'
+      });
+
+      this.mc.teardown(true, done);
+    });
+
+    it('should load a shared library by specifying a search path', function (done) {
+      const client = new MockClient();
+      this.mc = new AutoEncrypter(client, {
+        keyVaultNamespace: 'admin.datakeys',
+        logger: () => {},
+        kmsProviders: {
+          aws: { accessKeyId: 'example', secretAccessKey: 'example' },
+          local: { key: Buffer.alloc(96) }
+        },
+        extraOptions: {
+          cryptSharedLibSearchPaths: [path.dirname(sharedLibraryStub)]
+        }
+      });
+
+      expect(this.mc).to.not.have.property('_mongocryptdManager');
+      expect(this.mc).to.not.have.property('_mongocryptdClient');
+      expect(this.mc).to.have.deep.property('cryptSharedLibVersionInfo', {
+        // eslint-disable-next-line no-undef
+        version: BigInt(0x000600020001000),
+        versionStr: 'stubbed-crypt_shared'
+      });
+
+      this.mc.teardown(true, done);
     });
   });
 });
