@@ -36,6 +36,40 @@ describe('StateMachine', function () {
     }
   }
 
+  describe('#markCommand', function () {
+    let runCommandStub;
+    let dbStub;
+    let clientStub;
+
+    beforeEach(function () {
+      this.sinon = sinon.createSandbox();
+      runCommandStub = this.sinon.stub();
+      dbStub = this.sinon.createStubInstance(mongodb.Db, {
+        command: runCommandStub
+      });
+      clientStub = this.sinon.createStubInstance(mongodb.MongoClient, {
+        db: dbStub
+      });
+    });
+
+    const command = {
+      encryptedFields: {},
+      a: new BSON.Long('0'),
+      b: new BSON.Int32(0)
+    };
+    const options = { promoteLongs: false, promoteValues: false };
+    const serializedCommand = BSON.serialize(command);
+    const stateMachine = new StateMachine({ bson: BSON });
+    const callback = () => {};
+
+    context('when executing the command', function () {
+      it('does not promote values', function () {
+        stateMachine.markCommand(clientStub, 'test.coll', serializedCommand, callback);
+        expect(runCommandStub.calledWith(command, options)).to.be.true;
+      });
+    });
+  });
+
   describe('kmsRequest', function () {
     class MockSocket extends EventEmitter {
       constructor(callback) {
