@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
 import copy
 
 from pymongocrypt.binary import (MongoCryptBinaryIn,
@@ -25,6 +24,8 @@ from pymongocrypt.state_machine import MongoCryptCallback
 
 from pymongocrypt.crypto import (aes_256_cbc_encrypt,
                                  aes_256_cbc_decrypt,
+                                 aes_256_ctr_decrypt,
+                                 aes_256_ctr_encrypt,
                                  hmac_sha_256,
                                  hmac_sha_512,
                                  sha_256,
@@ -33,7 +34,9 @@ from pymongocrypt.crypto import (aes_256_cbc_encrypt,
 
 
 class MongoCryptOptions(object):
-    def __init__(self, kms_providers, schema_map=None):
+    def __init__(self, kms_providers, schema_map=None, encrypted_fields_map=None,
+                 bypass_query_analysis=False, crypt_shared_lib_path=None,
+                 crypt_shared_lib_required=False, bypass_encryption=False):
         """Options for :class:`MongoCrypt`.
 
         :Parameters:
@@ -63,6 +66,11 @@ class MongoCryptOptions(object):
             automatic encryption for client side encryption. Other validation
             rules in the JSON schema will not be enforced by the driver and
             will result in an error.
+          - `encrypted_fields_map`: Optional map encoded to BSON `bytes`.
+          - `bypass_query_analysis`: If ``True``, disable automatic analysis of
+            outgoing commands. Set `bypass_query_analysis` to use explicit
+            encryption on indexed fields without the MongoDB Enterprise Advanced
+            licensed crypt_shared library.
 
         .. versionadded:: 1.1
            Support for "azure" and "gcp" kms_providers.
@@ -133,11 +141,20 @@ class MongoCryptOptions(object):
         if schema_map is not None and not isinstance(schema_map, bytes):
             raise TypeError("schema_map must be bytes or None")
 
+        if encrypted_fields_map is not None and not isinstance(encrypted_fields_map, bytes):
+            raise TypeError("encrypted_fields_map must be bytes or None")
+
         self.kms_providers = kms_providers
         self.schema_map = schema_map
+        self.encrypted_fields_map = encrypted_fields_map
+        self.bypass_query_analysis = bypass_query_analysis
+        self.crypt_shared_lib_path = crypt_shared_lib_path
+        self.crypt_shared_lib_required = crypt_shared_lib_required
+        self.bypass_encryption = bypass_encryption
 
 
 class MongoCrypt(object):
+
     def __init__(self, options, callback):
         """Abstracts libmongocrypt's mongocrypt_t type.
 
@@ -145,7 +162,7 @@ class MongoCrypt(object):
           - `options`: A :class:`MongoCryptOptions`.
           - `callback`: A :class:`MongoCryptCallback`.
         """
-        self.__opts = options
+        self.__opts = options  # type: MongoCryptOptions
         self.__callback = callback
         self.__crypt = None
 
@@ -193,6 +210,16 @@ class MongoCrypt(object):
                         self.__crypt, binary_schema_map.bin):
                     self.__raise_from_status()
 
+        encrypted_fields_map = self.__opts.encrypted_fields_map
+        if encrypted_fields_map is not None:
+            with MongoCryptBinaryIn(encrypted_fields_map) as binary_encrypted_fields_map:
+                if not lib.mongocrypt_setopt_encrypted_field_config_map(
+                        self.__crypt, binary_encrypted_fields_map.bin):
+                    self.__raise_from_status()
+
+        if self.__opts.bypass_query_analysis:
+            lib.mongocrypt_setopt_bypass_query_analysis(self.__crypt)
+
         if not lib.mongocrypt_setopt_crypto_hooks(
                 self.__crypt, aes_256_cbc_encrypt, aes_256_cbc_decrypt,
                 secure_random, hmac_sha_512, hmac_sha_256, sha_256, ffi.NULL):
@@ -202,9 +229,27 @@ class MongoCrypt(object):
                 self.__crypt, sign_rsaes_pkcs1_v1_5, ffi.NULL):
             self.__raise_from_status()
 
+        if not lib.mongocrypt_setopt_aes_256_ctr(
+                self.__crypt, aes_256_ctr_encrypt, aes_256_ctr_decrypt, ffi.NULL):
+            self.__raise_from_status()
+
+        if self.__opts.crypt_shared_lib_path is not None:
+            lib.mongocrypt_setopt_set_crypt_shared_lib_path_override(
+                self.__crypt, self.__opts.crypt_shared_lib_path.encode("utf-8"))
+
+        if not self.__opts.bypass_encryption:
+            lib.mongocrypt_setopt_append_crypt_shared_lib_search_path(self.__crypt, b"$SYSTEM")
+
         if not lib.mongocrypt_init(self.__crypt):
             self.__raise_from_status()
 
+        if self.__opts.crypt_shared_lib_required and self.crypt_shared_lib_version is None:
+            raise MongoCryptError(
+                "crypt_shared_lib_required=True but the crypt_shared library could not be loaded "
+                "from crypt_shared_lib_path={}".format(
+                    self.__opts.crypt_shared_lib_path) +
+                " or the operating system's dynamic library search path")
+
     def __raise_from_status(self):
         status = lib.mongocrypt_status_new()
         try:
@@ -214,6 +259,13 @@ class MongoCrypt(object):
             lib.mongocrypt_status_destroy(status)
         raise exc
 
+    @property
+    def crypt_shared_lib_version(self):
+        ver = lib.mongocrypt_crypt_shared_lib_version_string(self.__crypt, ffi.NULL)
+        if ver == ffi.NULL:
+            return None
+        return ver
+
     def close(self):
         """Cleanup resources."""
         if self.__crypt is None:
@@ -449,13 +501,25 @@ class ExplicitEncryptionContext(MongoCryptContext):
 
             if opts.key_alt_name is not None:
                 with MongoCryptBinaryIn(opts.key_alt_name) as binary:
-                    if not lib.mongocrypt_ctx_setopt_key_alt_name(ctx,
-                                                                  binary.bin):
+                    if not lib.mongocrypt_ctx_setopt_key_alt_name(ctx, binary.bin):
                         self._raise_from_status()
 
+            if opts.index_key_id is not None:
+                with MongoCryptBinaryIn(opts.index_key_id) as binary:
+                    if not lib.mongocrypt_ctx_setopt_index_key_id(ctx, binary.bin):
+                        self._raise_from_status()
+
+            if opts.query_type is not None:
+                qt = str_to_bytes(opts.query_type)
+                if not lib.mongocrypt_ctx_setopt_query_type(ctx, qt, -1):
+                    self._raise_from_status()
+
+            if opts.contention_factor is not None:
+                if not lib.mongocrypt_ctx_setopt_contention_factor(ctx, opts.contention_factor):
+                    self._raise_from_status()
+
             with MongoCryptBinaryIn(value) as binary:
-                if not lib.mongocrypt_ctx_explicit_encrypt_init(ctx,
-                                                                binary.bin):
+                if not lib.mongocrypt_ctx_explicit_encrypt_init(ctx, binary.bin):
                     self._raise_from_status()
         except Exception:
             # Destroy the context on error.
