@@ -104,6 +104,14 @@ class TestMongoCryptOptions(unittest.TestCase):
             opts = MongoCryptOptions(kms_providers, schema_map)
             self.assertEqual(opts.kms_providers, kms_providers, msg=kms_providers)
             self.assertEqual(opts.schema_map, schema_map)
+            self.assertIsNone(opts.encrypted_fields_map)
+            self.assertFalse(opts.bypass_query_analysis)
+
+        encrypted_fields_map = bson_data('encrypted-field-config-map.json')
+        opts = MongoCryptOptions(valid[0][0], schema_map, encrypted_fields_map=encrypted_fields_map,
+                                 bypass_query_analysis=True)
+        self.assertEqual(opts.encrypted_fields_map, encrypted_fields_map)
+        self.assertTrue(opts.bypass_query_analysis)
 
     def test_mongocrypt_options_validation(self):
         with self.assertRaisesRegex(
@@ -132,8 +140,13 @@ class TestMongoCryptOptions(unittest.TestCase):
                 TypeError, "schema_map must be bytes or None"):
             MongoCryptOptions(valid_kms, schema_map={})
 
+        with self.assertRaisesRegex(
+                TypeError, "encrypted_fields_map must be bytes or None"):
+            MongoCryptOptions(valid_kms, encrypted_fields_map={})
+
 
 class TestMongoCrypt(unittest.TestCase):
+    maxDiff = None
 
     def test_mongocrypt(self):
         kms_providers = {
@@ -215,10 +228,10 @@ class TestMongoCrypt(unittest.TestCase):
             MongoCrypt(options, callback)
 
     @staticmethod
-    def create_mongocrypt():
+    def create_mongocrypt(**kwargs):
         return MongoCrypt(MongoCryptOptions({
             'aws': {'accessKeyId': 'example', 'secretAccessKey': 'example'},
-            'local': {'key': b'\x00'*96}}), MockCallback())
+            'local': {'key': b'\x00'*96}}, **kwargs), MockCallback())
 
     def _test_kms_context(self, ctx):
         key_filter = ctx.mongo_operation()
@@ -244,6 +257,8 @@ class TestMongoCrypt(unittest.TestCase):
     def test_encrypt(self):
         mc = self.create_mongocrypt()
         self.addCleanup(mc.close)
+        if mc.crypt_shared_lib_version is not None:
+            self.skipTest("this test must be skipped when crypt_shared is loaded")
         with mc.encryption_context('text', bson_data('command.json')) as ctx:
             self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_NEED_MONGO_COLLINFO)
 
@@ -256,6 +271,8 @@ class TestMongoCrypt(unittest.TestCase):
             self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_NEED_MONGO_MARKINGS)
 
             mongocryptd_cmd = ctx.mongo_operation()
+            self.assertEqual(bson.decode(mongocryptd_cmd, OPTS),
+                             json_data('mongocryptd-command.json'))
             self.assertEqual(mongocryptd_cmd,
                              bson_data('mongocryptd-command.json'))
 
@@ -290,6 +307,34 @@ class TestMongoCrypt(unittest.TestCase):
             self.assertEqual(encrypted, bson_data('command-reply.json'))
             self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_DONE)
 
+    def test_encrypt_encrypted_fields_map(self):
+        encrypted_fields_map = bson_data('compact/success/encrypted-field-config-map.json')
+        mc = self.create_mongocrypt(encrypted_fields_map=encrypted_fields_map)
+        self.addCleanup(mc.close)
+        with mc.encryption_context('db', bson_data('compact/success/cmd.json')) as ctx:
+            self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_NEED_MONGO_KEYS)
+
+            ctx.mongo_operation()
+            ctx.add_mongo_operation_result(bson_data(
+                'keys/12345678123498761234123456789012-local-document.json'))
+            self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_NEED_MONGO_KEYS)
+            ctx.mongo_operation()
+            ctx.add_mongo_operation_result(bson_data(
+                'keys/ABCDEFAB123498761234123456789012-local-document.json'))
+            self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_NEED_MONGO_KEYS)
+            ctx.mongo_operation()
+            ctx.add_mongo_operation_result(bson_data(
+                'keys/12345678123498761234123456789013-local-document.json'))
+            ctx.complete_mongo_operation()
+
+            self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_READY)
+
+            encrypted = ctx.finish()
+            self.assertEqual(bson.decode(encrypted, OPTS),
+                             json_data('compact/success/encrypted-payload.json'))
+            self.assertEqual(encrypted, bson_data('compact/success/encrypted-payload.json'))
+            self.assertEqual(ctx.state, lib.MONGOCRYPT_CTX_DONE)
+
 
 class MockCallback(MongoCryptCallback):
     def __init__(self,
@@ -327,6 +372,7 @@ class MockCallback(MongoCryptCallback):
 
 
 class TestMongoCryptCallback(unittest.TestCase):
+    maxDiff = None
 
     @staticmethod
     def mongo_crypt_opts():
@@ -334,6 +380,31 @@ class TestMongoCryptCallback(unittest.TestCase):
             'aws': {'accessKeyId': 'example', 'secretAccessKey': 'example'},
             'local': {'key': b'\x00'*96}})
 
+    @unittest.skipUnless(os.getenv("TEST_CRYPT_SHARED"), "this test requires TEST_CRYPT_SHARED=1")
+    def test_crypt_shared(self):
+        kms_providers = {
+            'aws': {'accessKeyId': 'example', 'secretAccessKey': 'example'},
+            'local': {'key': b'\x00'*96}}
+        mc = MongoCrypt(MongoCryptOptions(kms_providers), MockCallback())
+        self.addCleanup(mc.close)
+        self.assertIsNotNone(mc.crypt_shared_lib_version)
+        # Test that we can pick up crypt_shared automatically
+        encrypter = AutoEncrypter(MockCallback(), MongoCryptOptions(
+            kms_providers,
+            bypass_encryption=False,
+            crypt_shared_lib_required=True))
+        self.addCleanup(encrypter.close)
+        encrypter = AutoEncrypter(MockCallback(), MongoCryptOptions(
+            kms_providers,
+            crypt_shared_lib_path=os.environ["CRYPT_SHARED_PATH"],
+            crypt_shared_lib_required=True))
+        self.addCleanup(encrypter.close)
+        with self.assertRaisesRegex(MongoCryptError, "/doesnotexist"):
+            AutoEncrypter(MockCallback(), MongoCryptOptions(
+                kms_providers,
+                crypt_shared_lib_path="/doesnotexist",
+                crypt_shared_lib_required=True))
+
     def test_encrypt(self):
         encrypter = AutoEncrypter(MockCallback(
             list_colls_result=bson_data('collection-info.json'),
@@ -341,7 +412,7 @@ class TestMongoCryptCallback(unittest.TestCase):
             key_docs=[bson_data('key-document.json')],
             kms_reply=http_data('kms-reply.txt')), self.mongo_crypt_opts())
         self.addCleanup(encrypter.close)
-        encrypted = encrypter.encrypt('text', bson_data('command.json'))
+        encrypted = encrypter.encrypt('test', bson_data('command.json'))
         self.assertEqual(bson.decode(encrypted, OPTS),
                          json_data('encrypted-command.json'))
         self.assertEqual(encrypted, bson_data('encrypted-command.json'))
@@ -374,6 +445,7 @@ class KeyVaultCallback(MockCallback):
 
 
 class TestExplicitEncryption(unittest.TestCase):
+    maxDiff = None
 
     @staticmethod
     def mongo_crypt_opts():
@@ -401,7 +473,6 @@ class TestExplicitEncryption(unittest.TestCase):
         self.assertEqual(encoded_val, decrypted)
 
     def test_encrypt_decrypt(self):
-
         key_id = json_data('key-document.json')['_id']
         self._test_encrypt_decrypt(key_id=key_id)
 
@@ -409,6 +480,70 @@ class TestExplicitEncryption(unittest.TestCase):
         key_alt_name = json_data('key-document.json')['keyAltNames'][0]
         self._test_encrypt_decrypt(key_alt_name=key_alt_name)
 
+    def test_encrypt_errors(self):
+        key_id = json_data('key-document.json')['_id']
+        encrypter = ExplicitEncrypter(MockCallback(key_docs=[]), self.mongo_crypt_opts())
+        self.addCleanup(encrypter.close)
+
+        val = {'v': 'value123'}
+        encoded_val = bson.encode(val)
+        # Invalid algorithm.
+        with self.assertRaisesRegex(MongoCryptError, "algorithm"):
+            encrypter.encrypt(encoded_val, "Invalid", key_id)
+        # Invalid index_key_id type.
+        with self.assertRaises(TypeError):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, index_key_id=1)
+        with self.assertRaises(MongoCryptError):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, index_key_id=b'1234')
+        # Invalid query_type type.
+        with self.assertRaisesRegex(TypeError, "query_type"):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, query_type=42)
+        # Invalid query_type string.
+        with self.assertRaisesRegex(MongoCryptError, "query_type"):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, query_type='invalid query type string')
+        # Invalid contention_factor type.
+        with self.assertRaisesRegex(TypeError, "contention_factor"):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, contention_factor='not an int')
+        with self.assertRaisesRegex(MongoCryptError, "contention"):
+            encrypter.encrypt(encoded_val, "Indexed", key_id, contention_factor=-1)
+        # Invalid: Unindexed + query_type is an error.
+        with self.assertRaisesRegex(MongoCryptError, "query"):
+            encrypter.encrypt(encoded_val, "Unindexed", key_id, query_type='equality')
+        # Invalid: Unindexed + contention_factor is an error.
+        with self.assertRaisesRegex(MongoCryptError, "contention"):
+            encrypter.encrypt(encoded_val, "Unindexed", key_id, contention_factor=1)
+
+    def test_encrypt_indexed(self):
+        key_path = 'keys/ABCDEFAB123498761234123456789012-local-document.json'
+        index_key_path = 'keys/12345678123498761234123456789012-local-document.json'
+        key_id = json_data(key_path)['_id']
+        index_key_id = json_data(index_key_path)['_id']
+        encrypter = ExplicitEncrypter(MockCallback(
+            key_docs=[bson_data(key_path), bson_data(index_key_path)],
+            kms_reply=http_data('kms-reply.txt')), self.mongo_crypt_opts())
+        self.addCleanup(encrypter.close)
+
+        val = {'v': 'value123'}
+        encoded_val = bson.encode(val)
+        for kwargs in [
+            dict(algorithm='Indexed', contention_factor=0),
+            dict(algorithm='Indexed', query_type='equality', contention_factor=0),
+            dict(algorithm='Indexed', contention_factor=100),
+            dict(algorithm='Unindexed'),
+        ]:
+            kwargs['key_id'] = key_id
+            kwargs['index_key_id'] = index_key_id
+            encrypted = encrypter.encrypt(encoded_val, **kwargs)
+            encrypted_val = bson.decode(encrypted, OPTS)['v']
+            self.assertIsInstance(encrypted_val, Binary)
+            self.assertEqual(encrypted_val.subtype, 6)
+
+            # Queryable Encryption find payloads cannot be round-tripped.
+            if 'query_type' not in kwargs:
+                decrypted = encrypter.decrypt(encrypted)
+                self.assertEqual(bson.decode(decrypted, OPTS), val)
+                self.assertEqual(encoded_val, decrypted)
+
     def test_data_key_creation(self):
         mock_key_vault = KeyVaultCallback(
             kms_reply=http_data('kms-encrypt-reply.txt'))
@@ -453,7 +588,11 @@ def read(filename, **kwargs):
 OPTS = CodecOptions(uuid_representation=UuidRepresentation.UNSPECIFIED)
 
 # Use SON to preserve the order of fields while parsing json.
-JSON_OPTS = JSONOptions(document_class=SON,
+if sys.version_info[:2] < (3, 6):
+    document_class = SON
+else:
+    document_class = dict
+JSON_OPTS = JSONOptions(document_class=document_class,
                         uuid_representation=UuidRepresentation.UNSPECIFIED)
 
 
