@@ -102,12 +102,14 @@ static char invalid_utf8[] = {(char) 0x80, (char) 0x00};
 #define EX_DECRYPT_INIT_FAILS(bin, msg) \
    ASSERT_FAILS (mongocrypt_ctx_explicit_decrypt_init (ctx, bin), ctx, msg);
 
-#define REFRESH                  \
-   mongocrypt_ctx_destroy (ctx); \
-   ctx = mongocrypt_ctx_new (crypt);
+#define REFRESH                         \
+   do {                                 \
+      mongocrypt_ctx_destroy (ctx);     \
+      ctx = mongocrypt_ctx_new (crypt); \
+   } while (0)
 
-#define DET "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic"
-#define RAND "AEAD_AES_256_CBC_HMAC_SHA_512-Random"
+#define DET MONGOCRYPT_ALGORITHM_DETERMINISTIC_STR
+#define RAND MONGOCRYPT_ALGORITHM_RANDOM_STR
 
 /* Test valid and invalid options */
 static void
@@ -116,7 +118,7 @@ _test_setopt_masterkey_aws (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    REFRESH;
    MASTERKEY_AWS_FAILS (NULL, 0, "cmk", 3, "invalid region");
@@ -161,7 +163,7 @@ _test_setopt_masterkey_local (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    /* Test double setting. */
    REFRESH;
@@ -187,7 +189,7 @@ _test_setopt_key_encryption_key_azure (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    /* Test double setting. */
    REFRESH;
@@ -220,7 +222,7 @@ _test_setopt_key_encryption_key_gcp (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    /* Test double setting. */
    REFRESH;
@@ -258,7 +260,7 @@ _test_setopt_key_id (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    /* Test double setting. */
    REFRESH;
@@ -291,7 +293,7 @@ _test_setopt_key_alt_name (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    /* Test double setting - actually succeeds since multiple key alt names
     * allowed for data keys. */
@@ -338,7 +340,8 @@ _test_setopt_key_material (_mongocrypt_tester_t *tester)
       "{'keyMaterial': {'$binary': {'base64': '%s', 'subType': '00'}}%s}";
    mongocrypt_binary_t *const valid = TEST_BSON (pattern, material, "");
 
-   mongocrypt_t *crypt = _mongocrypt_tester_mongocrypt ();
+   mongocrypt_t *crypt =
+      _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    mongocrypt_ctx_t *ctx = NULL;
 
    /* Test double setting. */
@@ -411,7 +414,7 @@ _test_setopt_algorithm (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    REFRESH;
    ALGORITHM_FAILS (DET, -2, "invalid algorithm length");
@@ -452,7 +455,7 @@ _test_setopt_for_datakey (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_binary_t *uuid;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    uuid = TEST_BIN (16);
 
    /* Test required and prohibited options. */
@@ -506,7 +509,7 @@ _test_setopt_for_datakey (_mongocrypt_tester_t *tester)
 
    REFRESH;
    MASTERKEY_AWS_OK ("region", -1, "cmk", -1);
-   ALGORITHM_OK ("AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic", -1);
+   ALGORITHM_OK (MONGOCRYPT_ALGORITHM_DETERMINISTIC_STR, -1);
    DATAKEY_INIT_FAILS ("algorithm prohibited");
 
    /* Test setting options after init. */
@@ -537,7 +540,7 @@ _test_setopt_for_encrypt (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_binary_t *uuid, *cmd;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    cmd = TEST_FILE ("./test/example/cmd.json");
    uuid = TEST_BIN (16);
 
@@ -598,7 +601,7 @@ _test_setopt_for_explicit_encrypt (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_binary_t *bson, *uuid;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    uuid = TEST_BIN (16);
    bson = TEST_BSON ("{'v': 'hello'}");
 
@@ -659,7 +662,7 @@ _test_setopt_for_explicit_encrypt (_mongocrypt_tester_t *tester)
 
    REFRESH;
    KEY_ID_OK (uuid);
-   EX_ENCRYPT_INIT_FAILS (bson, "algorithm required");
+   EX_ENCRYPT_INIT_FAILS (bson, "algorithm or index type required");
 
    REFRESH;
    ALGORITHM_OK (RAND, -1);
@@ -696,6 +699,121 @@ _test_setopt_for_explicit_encrypt (_mongocrypt_tester_t *tester)
    ALGORITHM_FAILS ("bad-algo", -1, "unsupported algorithm");
    EX_ENCRYPT_INIT_FAILS (bson, "unsupported algorithm");
 
+   /* It is an error to set the FLE 1 keyAltName option with any of the FLE 2
+    * options (index_type, index_key_id, contention_factor, or query_type). */
+   {
+      REFRESH;
+      KEY_ALT_NAME_OK (TEST_BSON ("{'keyAltName': 'abc'}"));
+      ASSERT_OK (mongocrypt_ctx_setopt_algorithm (
+                    ctx, MONGOCRYPT_ALGORITHM_UNINDEXED_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set both key alt name and index type");
+
+      REFRESH;
+      KEY_ALT_NAME_OK (TEST_BSON ("{'keyAltName': 'abc'}"));
+      ASSERT_OK (mongocrypt_ctx_setopt_index_key_id (ctx, uuid), ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set both key alt name and index key id");
+
+      REFRESH;
+      KEY_ALT_NAME_OK (TEST_BSON ("{'keyAltName': 'abc'}"));
+      ASSERT_OK (mongocrypt_ctx_setopt_contention_factor (ctx, 123), ctx);
+      EX_ENCRYPT_INIT_FAILS (
+         bson, "cannot set both key alt name and contention factor");
+
+      REFRESH;
+      KEY_ALT_NAME_OK (TEST_BSON ("{'keyAltName': 'abc'}"));
+      ASSERT_OK (mongocrypt_ctx_setopt_query_type (
+                    ctx, MONGOCRYPT_QUERY_TYPE_EQUALITY_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set both key alt name and query type");
+   }
+
+   /* It is an error to set the FLE 1 algorithm option with any of the FLE 2
+    * options (index_type, index_key_id, contention_factor, or query_type). */
+   {
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ALGORITHM_OK (RAND, -1);
+      ASSERT_OK (mongocrypt_ctx_setopt_index_key_id (ctx, uuid), ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set both algorithm and index key id");
+
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ALGORITHM_OK (RAND, -1);
+      ASSERT_OK (mongocrypt_ctx_setopt_contention_factor (ctx, 123), ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set both algorithm and contention factor");
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ALGORITHM_OK (RAND, -1);
+      ASSERT_OK (mongocrypt_ctx_setopt_query_type (
+                    ctx, MONGOCRYPT_QUERY_TYPE_EQUALITY_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson, "cannot set both algorithm and query type");
+   }
+
+   /* Require either index_type or algorithm */
+   {
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      EX_ENCRYPT_INIT_FAILS (bson, "algorithm or index type required");
+   }
+
+   /* It is an error to set contention_factor with index_type ==
+    * MONGOCRYPT_INDEX_TYPE_NONE */
+   {
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ASSERT_OK (mongocrypt_ctx_setopt_contention_factor (ctx, 0), ctx);
+      ASSERT_OK (mongocrypt_ctx_setopt_algorithm (
+                    ctx, MONGOCRYPT_ALGORITHM_UNINDEXED_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson,
+                             "cannot set contention factor with no index type");
+   }
+
+   /* It is an error to set query_type with index_type ==
+    * MONGOCRYPT_INDEX_TYPE_NONE */
+   {
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ASSERT_OK (mongocrypt_ctx_setopt_query_type (
+                    ctx, MONGOCRYPT_QUERY_TYPE_EQUALITY_STR, -1),
+                 ctx);
+      ASSERT_OK (mongocrypt_ctx_setopt_algorithm (
+                    ctx, MONGOCRYPT_ALGORITHM_UNINDEXED_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson, "cannot set query type with no index type");
+   }
+
+   /* Contention factor is required for "Indexed" algorithm. */
+   {
+      REFRESH;
+      /* Set key ID to get past the 'either key id or key alt name required'
+       * error */
+      KEY_ID_OK (uuid);
+      ASSERT_OK (mongocrypt_ctx_setopt_algorithm (
+                    ctx, MONGOCRYPT_ALGORITHM_INDEXED_STR, -1),
+                 ctx);
+      EX_ENCRYPT_INIT_FAILS (bson, "contention factor is required");
+   }
+
    mongocrypt_ctx_destroy (ctx);
    mongocrypt_destroy (crypt);
 }
@@ -708,7 +826,7 @@ _test_setopt_for_decrypt (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_binary_t *bson, *uuid;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    uuid = TEST_BIN (16);
    bson = TEST_BSON ("{'a': 1}");
 
@@ -766,7 +884,7 @@ _test_setopt_for_explicit_decrypt (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_binary_t *bson, *uuid;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    uuid = TEST_BIN (16);
    bson = TEST_FILE ("./test/data/explicit-decryption-input.json");
 
@@ -819,7 +937,7 @@ _test_setopt_failure_uninitialized (_mongocrypt_tester_t *tester)
    mongocrypt_ctx_t *ctx = NULL;
    mongocrypt_status_t *status;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
    status = mongocrypt_status_new ();
 
    REFRESH;
@@ -843,7 +961,7 @@ _test_setopt_endpoint (_mongocrypt_tester_t *tester)
    mongocrypt_t *crypt;
    mongocrypt_ctx_t *ctx = NULL;
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    REFRESH;
    ENDPOINT_FAILS ("example.com", -2, "Invalid endpoint");
