@@ -25,6 +25,9 @@
 #include "mongocrypt-marking-private.h"
 #include "test-mongocrypt.h"
 
+#ifdef MONGOCRYPT_ENABLE_CRYPTO_COMMON_CRYPTO
+#include <sys/sysctl.h>
+#endif
 
 /* Return a repeated character with no null terminator. */
 char *
@@ -62,6 +65,13 @@ _load_json_as_bson (const char *path, bson_t *out)
    bson_json_reader_destroy (reader);
 }
 
+#define TEST_DATA_COUNT_INC(var)                                               \
+   (var)++;                                                                    \
+   if ((var) >= TEST_DATA_COUNT) {                                             \
+      TEST_ERROR (                                                             \
+         "TEST_DATA_COUNT exceeded for %s. Increment TEST_DATA_COUNT.", #var); \
+   }
+
 static void
 _load_json (_mongocrypt_tester_t *tester, const char *path)
 {
@@ -73,7 +83,7 @@ _load_json (_mongocrypt_tester_t *tester, const char *path)
    buf = &tester->file_bufs[tester->file_count];
    _mongocrypt_buffer_steal_from_bson (buf, &as_bson);
    tester->file_paths[tester->file_count] = bson_strdup (path);
-   tester->file_count++;
+   TEST_DATA_COUNT_INC (tester->file_count);
 }
 
 
@@ -123,7 +133,7 @@ _load_http (_mongocrypt_tester_t *tester, const char *path)
 
    bson_free (contents);
    tester->file_paths[tester->file_count] = bson_strdup (path);
-   tester->file_count++;
+   TEST_DATA_COUNT_INC (tester->file_count);
 }
 
 
@@ -153,7 +163,7 @@ _mongocrypt_tester_install (_mongocrypt_tester_t *tester,
 
    tester->test_fns[tester->test_count] = fn;
    tester->test_names[tester->test_count] = bson_strdup (name);
-   tester->test_count++;
+   TEST_DATA_COUNT_INC (tester->test_count);
 }
 
 
@@ -164,7 +174,8 @@ _mongocrypt_tester_file (_mongocrypt_tester_t *tester, const char *path)
    mongocrypt_binary_t *to_return;
 
    to_return = mongocrypt_binary_new ();
-   tester->test_bin[tester->bin_count++] = to_return;
+   tester->test_bin[tester->bin_count] = to_return;
+   TEST_DATA_COUNT_INC (tester->bin_count);
 
    for (i = 0; i < tester->file_count; i++) {
       if (0 == strcmp (tester->file_paths[i], path)) {
@@ -207,7 +218,8 @@ _mongocrypt_tester_bson_from_json (_mongocrypt_tester_t *tester,
    }
 
    va_end (ap);
-   bson = &tester->test_bson[tester->bson_count++];
+   bson = &tester->test_bson[tester->bson_count];
+   TEST_DATA_COUNT_INC (tester->bson_count);
    if (!bson_init_from_json (bson, full_json, strlen (full_json), &error)) {
       fprintf (stderr, "%s", error.message);
       abort ();
@@ -239,13 +251,15 @@ _mongocrypt_tester_bin_from_json (_mongocrypt_tester_t *tester,
    }
 
    va_end (ap);
-   bson = &tester->test_bson[tester->bson_count++];
+   bson = &tester->test_bson[tester->bson_count];
+   TEST_DATA_COUNT_INC (tester->bson_count);
    if (!bson_init_from_json (bson, full_json, strlen (full_json), &error)) {
-      fprintf (stderr, "%s", error.message);
+      fprintf (stderr, "failed to parse JSON %s: %s", error.message, json);
       abort ();
    }
    bin = mongocrypt_binary_new ();
-   tester->test_bin[tester->bin_count++] = bin;
+   tester->test_bin[tester->bin_count] = bin;
+   TEST_DATA_COUNT_INC (tester->bin_count);
    bin->data = (uint8_t *) bson_get_data (bson);
    bin->len = bson->len;
    bson_free (full_json);
@@ -272,8 +286,10 @@ _mongocrypt_tester_bin (_mongocrypt_tester_t *tester, int size)
 
    bin = mongocrypt_binary_new_from_data (blob, size);
 
-   tester->test_blob[tester->blob_count++] = blob;
-   tester->test_bin[tester->bin_count++] = bin;
+   tester->test_blob[tester->blob_count] = blob;
+   TEST_DATA_COUNT_INC (tester->blob_count);
+   tester->test_bin[tester->bin_count] = bin;
+   TEST_DATA_COUNT_INC (tester->bin_count);
    return bin;
 }
 
@@ -310,22 +326,30 @@ _mongocrypt_tester_run_ctx_to (_mongocrypt_tester_t *tester,
    while (state != stop_state) {
       switch (state) {
       case MONGOCRYPT_CTX_NEED_MONGO_COLLINFO:
+         if (tester->paths.collection_info) {
+            bin = TEST_FILE (tester->paths.collection_info);
+         } else {
+            bin = TEST_FILE ("./test/example/collection-info.json");
+         }
          BSON_ASSERT (ctx->type == _MONGOCRYPT_TYPE_ENCRYPT);
-         BSON_ASSERT (mongocrypt_ctx_mongo_feed (
-            ctx, TEST_FILE ("./test/example/collection-info.json")));
+         BSON_ASSERT (mongocrypt_ctx_mongo_feed (ctx, bin));
          BSON_ASSERT (mongocrypt_ctx_mongo_done (ctx));
          break;
       case MONGOCRYPT_CTX_NEED_MONGO_MARKINGS:
+         if (tester->paths.mongocryptd_reply) {
+            bin = TEST_FILE (tester->paths.mongocryptd_reply);
+         } else {
+            bin = TEST_FILE ("./test/example/mongocryptd-reply.json");
+         }
          BSON_ASSERT (ctx->type == _MONGOCRYPT_TYPE_ENCRYPT);
-         res = mongocrypt_ctx_mongo_feed (
-            ctx, TEST_FILE ("./test/example/mongocryptd-reply.json"));
+         res = mongocrypt_ctx_mongo_feed (ctx, bin);
          mongocrypt_ctx_status (ctx, status);
          ASSERT_OR_PRINT (res, status);
          BSON_ASSERT (mongocrypt_ctx_mongo_done (ctx));
          break;
       case MONGOCRYPT_CTX_NEED_MONGO_KEYS:
-         if (tester->key_file_path) {
-            bin = TEST_FILE (tester->key_file_path);
+         if (tester->paths.key_file) {
+            bin = TEST_FILE (tester->paths.key_file);
          } else {
             bin = TEST_FILE ("./test/example/key-document.json");
          }
@@ -367,6 +391,8 @@ _mongocrypt_tester_run_ctx_to (_mongocrypt_tester_t *tester,
          ASSERT_STATE_EQUAL (state, stop_state);
          mongocrypt_status_destroy (status);
          return;
+      default:
+         BSON_ASSERT (false && "Invalid state");
       }
       state = mongocrypt_ctx_state (ctx);
    }
@@ -413,7 +439,7 @@ _mongocrypt_tester_encrypted_doc (_mongocrypt_tester_t *tester)
       return bin;
    }
 
-   crypt = _mongocrypt_tester_mongocrypt ();
+   crypt = _mongocrypt_tester_mongocrypt (TESTER_MONGOCRYPT_DEFAULT);
 
    ctx = mongocrypt_ctx_new (crypt);
    ASSERT_OK (mongocrypt_ctx_encrypt_init (
@@ -478,7 +504,7 @@ _mongocrypt_tester_fill_buffer (_mongocrypt_buffer_t *buf, int n)
    "I4VnrEMGrQ8e+qYSwYk9Gh6dKGoRMAPYVXQAO0fIsHF/T0a"
 
 mongocrypt_t *
-_mongocrypt_tester_mongocrypt (void)
+_mongocrypt_tester_mongocrypt (tester_mongocrypt_flags flags)
 {
    mongocrypt_t *crypt;
    char localkey_data[MONGOCRYPT_KEY_LEN] = {0};
@@ -519,7 +545,17 @@ _mongocrypt_tester_mongocrypt (void)
    ASSERT_OK (mongocrypt_setopt_kms_providers (crypt, bin), crypt);
    bson_destroy (kms_providers);
    mongocrypt_binary_destroy (bin);
+   if (flags & TESTER_MONGOCRYPT_WITH_CRYPT_SHARED_LIB) {
+      mongocrypt_setopt_append_crypt_shared_lib_search_path (crypt, "$ORIGIN");
+   }
    ASSERT_OK (mongocrypt_init (crypt), crypt);
+   if (flags & TESTER_MONGOCRYPT_WITH_CRYPT_SHARED_LIB) {
+      if (mongocrypt_crypt_shared_lib_version (crypt) == 0) {
+         BSON_ASSERT (
+            !"tester mongocrypt requested WITH_CRYPT_SHARED_LIB, but no "
+             "crypt_shared library was loaded by mongocrypt_init");
+      }
+   }
    return crypt;
 }
 
@@ -634,6 +670,101 @@ _test_setopt_schema (_mongocrypt_tester_t *tester)
    mongocrypt_destroy (crypt);
 }
 
+static void
+_test_setopt_encrypted_field_config_map (_mongocrypt_tester_t *tester)
+{
+   mongocrypt_t *crypt;
+
+   /* Test success. */
+   crypt = mongocrypt_new ();
+   ASSERT_OK (
+      mongocrypt_setopt_kms_providers (
+         crypt,
+         TEST_BSON (
+            "{'aws': {'accessKeyId': 'foo', 'secretAccessKey': 'bar'}}")),
+      crypt);
+   ASSERT_OK (
+      mongocrypt_setopt_encrypted_field_config_map (
+         crypt, TEST_FILE ("./test/data/encrypted-field-config-map.json")),
+      crypt);
+   ASSERT_OK (mongocrypt_init (crypt), crypt);
+   mongocrypt_destroy (crypt);
+
+   /* Test double setting. */
+   crypt = mongocrypt_new ();
+   ASSERT_OK (
+      mongocrypt_setopt_encrypted_field_config_map (
+         crypt, TEST_FILE ("./test/data/encrypted-field-config-map.json")),
+      crypt);
+   ASSERT_FAILS (
+      mongocrypt_setopt_encrypted_field_config_map (
+         crypt, TEST_FILE ("./test/data/encrypted-field-config-map.json")),
+      crypt,
+      "already set encrypted_field_config_map");
+   mongocrypt_destroy (crypt);
+
+   /* Test NULL/empty input */
+   crypt = mongocrypt_new ();
+   ASSERT_FAILS (mongocrypt_setopt_encrypted_field_config_map (crypt, NULL),
+                 crypt,
+                 "passed null encrypted_field_config_map");
+   mongocrypt_destroy (crypt);
+
+   crypt = mongocrypt_new ();
+   ASSERT_FAILS (
+      mongocrypt_setopt_encrypted_field_config_map (crypt, TEST_BIN (0)),
+      crypt,
+      "passed null encrypted_field_config_map");
+   mongocrypt_destroy (crypt);
+
+   /* Test malformed BSON */
+   crypt = mongocrypt_new ();
+   ASSERT_FAILS (
+      mongocrypt_setopt_encrypted_field_config_map (crypt, TEST_BIN (10)),
+      crypt,
+      "invalid bson");
+   mongocrypt_destroy (crypt);
+
+   /* Test that it is OK to set both the encrypted field config map and schema
+    * map if there are no intersecting collections. */
+   crypt = mongocrypt_new ();
+   ASSERT_OK (mongocrypt_setopt_schema_map (
+                 crypt, TEST_BSON ("{'db.coll1': {}, 'db.coll2': {}}")),
+              crypt);
+   ASSERT_OK (mongocrypt_setopt_encrypted_field_config_map (
+                 crypt, TEST_BSON ("{'db.coll3': {}, 'db.coll3': {}}")),
+              crypt);
+   ASSERT_OK (
+      mongocrypt_setopt_kms_providers (
+         crypt,
+         TEST_BSON (
+            "{'aws': {'accessKeyId': 'foo', 'secretAccessKey': 'bar'}}")),
+      crypt);
+   ASSERT_OK (mongocrypt_init (crypt), crypt);
+   mongocrypt_destroy (crypt);
+
+   /* Test that it is an error to set both the encrypted field config map and
+    * schema map referencing the same collection. */
+   crypt = mongocrypt_new ();
+   ASSERT_OK (mongocrypt_setopt_schema_map (
+                 crypt, TEST_BSON ("{'db.coll1': {}, 'db.coll2': {}}")),
+              crypt);
+   ASSERT_OK (mongocrypt_setopt_encrypted_field_config_map (
+                 crypt, TEST_BSON ("{'db.coll1': {}, 'db.coll3': {}}")),
+              crypt);
+   ASSERT_OK (
+      mongocrypt_setopt_kms_providers (
+         crypt,
+         TEST_BSON (
+            "{'aws': {'accessKeyId': 'foo', 'secretAccessKey': 'bar'}}")),
+      crypt);
+   ASSERT_FAILS (
+      mongocrypt_init (crypt),
+      crypt,
+      "db.coll1 is present in both schema_map and encrypted_field_config_map");
+   mongocrypt_destroy (crypt);
+}
+
 static void
 _test_setopt_invalid_kms_providers (_mongocrypt_tester_t *tester)
 {
@@ -729,13 +860,12 @@ _test_setopt_kms_providers (_mongocrypt_tester_t *tester)
       {"{'kmip': {'endpoint': 'localhost' }}", NULL},
       {"{'kmip': {'endpoint': '127.0.0.1:5696', 'extra': 'invalid' }}",
        "Unexpected field: 'extra'"},
-       /* Empty documents are OK for on-demand KMS credentials */
+      /* Empty documents are OK for on-demand KMS credentials */
       {"{'aws': {}}", NULL},
       {"{'azure': {}}", NULL},
       {"{'local': {}}", NULL},
       {"{'gcp': {}}", NULL},
-      {"{'kmip': {}}", NULL}
-       };
+      {"{'kmip': {}}", NULL}};
 
    for (i = 0; i < sizeof (tests) / sizeof (tests[0]); i++) {
       mongocrypt_t *crypt;
@@ -756,6 +886,8 @@ _test_setopt_kms_providers (_mongocrypt_tester_t *tester)
    }
 }
 
+bool _aes_ctr_is_supported_by_os = true;
+
 int
 main (int argc, char **argv)
 {
@@ -771,6 +903,7 @@ main (int argc, char **argv)
    _mongocrypt_tester_install_data_key (&tester);
    _mongocrypt_tester_install_ctx_encrypt (&tester);
    _mongocrypt_tester_install_ctx_decrypt (&tester);
+   _mongocrypt_tester_install_ctx_rewrap_many_datakey (&tester);
    _mongocrypt_tester_install_ciphertext (&tester);
    _mongocrypt_tester_install_key_broker (&tester);
    _mongocrypt_tester_install (&tester,
@@ -786,6 +919,10 @@ main (int argc, char **argv)
    _mongocrypt_tester_install_traverse_util (&tester);
    _mongocrypt_tester_install (
       &tester, "_test_setopt_schema", _test_setopt_schema, CRYPTO_REQUIRED);
+   _mongocrypt_tester_install (&tester,
+                               "_test_setopt_encrypted_field_config_map",
+                               _test_setopt_encrypted_field_config_map,
+                               CRYPTO_REQUIRED);
    _mongocrypt_tester_install (&tester,
                                "_test_setopt_invalid_kms_providers",
                                _test_setopt_invalid_kms_providers,
@@ -804,7 +941,39 @@ main (int argc, char **argv)
    _mongocrypt_tester_install_kms_ctx (&tester);
    _mongocrypt_tester_install_csfle_lib (&tester);
    _mongocrypt_tester_install_dll (&tester);
+   _mongocrypt_tester_install_mc_tokens (&tester);
+   _mongocrypt_tester_install_fle2_payloads (&tester);
+   _mongocrypt_tester_install_efc (&tester);
+   _mongocrypt_tester_install_compact (&tester);
+   _mongocrypt_tester_install_fle2_payload_uev (&tester);
+   _mongocrypt_tester_install_fle2_payload_iup (&tester);
 
+#ifdef MONGOCRYPT_ENABLE_CRYPTO_COMMON_CRYPTO
+   char osversion[32];
+   size_t osversion_len = sizeof (osversion) - 1;
+   int osversion_name[] = {CTL_KERN, KERN_OSRELEASE};
+
+   _aes_ctr_is_supported_by_os = false;
+
+   if (sysctl (osversion_name, 2, osversion, &osversion_len, NULL, 0) == -1) {
+      goto get_os_version_failed;
+   }
+
+   uint32_t major, minor;
+   if (sscanf (osversion, "%u.%u", &major, &minor) != 2) {
+      goto get_os_version_failed;
+   }
+
+   if (major >= 20) {
+      // macOS 11 and newer
+      _aes_ctr_is_supported_by_os = true;
+   } else {
+      major -= 4;
+      // macOS 10.1.1 and newer. CTR unsupported in 10.14 and earlier
+      _aes_ctr_is_supported_by_os = major > 14;
+   }
+get_os_version_failed:
+#endif
 
    printf ("Running tests...\n");
    for (i = 0; tester.test_names[i]; i++) {
@@ -824,7 +993,7 @@ main (int argc, char **argv)
       printf ("  begin %s\n", tester.test_names[i]);
       tester.test_fns[i](&tester);
       /* Clear state. */
-      tester.key_file_path = NULL;
+      memset (&tester.paths, 0, sizeof (tester.paths));
       printf ("  end %s\n", tester.test_names[i]);
    }
    printf ("... done running tests\n");
