@@ -33,9 +33,18 @@
 struct __mongocrypt_tester_t;
 typedef void (*_mongocrypt_test_fn) (struct __mongocrypt_tester_t *tester);
 
-/* Arbitrary max of 1024 instances of temporary test data. Increase as needed.
+typedef enum tester_mongocrypt_flags {
+   /// Default settings for creating a mongocrypt_t for testing
+   TESTER_MONGOCRYPT_DEFAULT = 0,
+   /// Create a mongocrypt_t that has the crypt_shared library loaded. A
+   /// crypt_shared library must be present in the same directory as the test
+   /// executable.
+   TESTER_MONGOCRYPT_WITH_CRYPT_SHARED_LIB = 1 << 0,
+} tester_mongocrypt_flags;
+
+/* Arbitrary max of 2048 instances of temporary test data. Increase as needed.
  */
-#define TEST_DATA_COUNT 1024
+#define TEST_DATA_COUNT 2048
 typedef struct __mongocrypt_tester_t {
    int test_count;
    char *test_names[TEST_DATA_COUNT];
@@ -54,7 +63,12 @@ typedef struct __mongocrypt_tester_t {
    int blob_count;
    uint8_t *test_blob[TEST_DATA_COUNT];
 
-   char *key_file_path;
+   // Overides used in run_ctx_to
+   struct {
+      char *collection_info;
+      char *mongocryptd_reply;
+      char *key_file;
+   } paths;
 
    /* Example encrypted doc. */
    _mongocrypt_buffer_t encrypted_doc;
@@ -91,7 +105,7 @@ _mongocrypt_tester_fill_buffer (_mongocrypt_buffer_t *buf, int n);
 
 /* Return a new initialized mongocrypt_t for testing. */
 mongocrypt_t *
-_mongocrypt_tester_mongocrypt (void);
+_mongocrypt_tester_mongocrypt (tester_mongocrypt_flags options);
 
 typedef enum {
    CRYPTO_REQUIRED,
@@ -134,6 +148,11 @@ void
 _mongocrypt_tester_install_ctx_decrypt (_mongocrypt_tester_t *tester);
 
 
+void
+_mongocrypt_tester_install_ctx_rewrap_many_datakey (
+   _mongocrypt_tester_t *tester);
+
+
 void
 _mongocrypt_tester_install_ciphertext (_mongocrypt_tester_t *tester);
 
@@ -191,6 +210,24 @@ _mongocrypt_tester_install_cache_oauth (_mongocrypt_tester_t *tester);
 void
 _mongocrypt_tester_install_kms_ctx (_mongocrypt_tester_t *tester);
 
+void
+_mongocrypt_tester_install_mc_tokens (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_fle2_payloads (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_efc (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_compact (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_fle2_payload_uev (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_fle2_payload_iup (_mongocrypt_tester_t *tester);
+
 /* Conveniences for getting test data. */
 
 /* Get a temporary bson_t from a JSON string. Do not free it. */
@@ -226,5 +263,6 @@ _mongocrypt_tester_file (_mongocrypt_tester_t *tester, const char *path);
 void
 _load_json_as_bson (const char *path, bson_t *out);
 
+extern bool _aes_ctr_is_supported_by_os;
 
 #endif
